/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsISimpleEnumerator;

// to be implemented by an embeddor that wants to override some strings
[scriptable, uuid(965eb278-5678-456b-82a7-20a0c86a803c)]
interface nsIStringBundleOverride : nsISupports
{
  /**
   * get the override value for a particular key in a particular
   * string bundle
   */
  AString getStringFromName(in AUTF8String url,
                            in ACString key);

  /**
   * get all override keys for a given string bundle
   */
  nsISimpleEnumerator enumerateKeysInBundle(in AUTF8String url);
};
